/*
 *  Abstraction layer for MIXER
 *  Copyright (c) by Jaroslav Kysela <perex@jcu.cz>
 */
 
#define ULTRA_MIXER_DEVS	16		/* max mixer devices per card */

#define ULTRA_MIXER_CAP_EXCL_INPUT      0x00000001      /* only one recording source at a time */

#define ULTRA_MIXER_OSSDEVS	17
#define ULTRA_MIXER_VOLUME	0
#define ULTRA_MIXER_BASS	1
#define ULTRA_MIXER_TREBLE	2
#define ULTRA_MIXER_SYNTH	3
#define ULTRA_MIXER_PCM		4
#define ULTRA_MIXER_SPEAKER	5
#define ULTRA_MIXER_LINE	6
#define ULTRA_MIXER_MIC		7
#define ULTRA_MIXER_CD		8
#define ULTRA_MIXER_IMIX	9		/* recording monitor */
#define ULTRA_MIXER_ALTPCM	10
#define ULTRA_MIXER_RECLEV	11		/* recording level */
#define ULTRA_MIXER_IGAIN	12		/* input gain */
#define ULTRA_MIXER_OGAIN	13		/* output gain */
#define ULTRA_MIXER_LINE1	14
#define ULTRA_MIXER_LINE2	15
#define ULTRA_MIXER_LINE3	16
#define ULTRA_MIXER_UNKNOWN	(32+1)

struct oss_mixer_info {
  char id[ 16 ];
  char name[ 32 ];
};
    
#define ULTRA_MIXER_SET_RECSRC	_IOWR( 'M', 255, int )
#define ULTRA_MIXER_RECSRC	_IOR ( 'M', 255, int )
#define ULTRA_MIXER_DEVMASK	_IOR ( 'M', 254, int )
#define ULTRA_MIXER_RECMASK	_IOR ( 'M', 253, int )
#define ULTRA_MIXER_CAPS	_IOR ( 'M', 252, int )
#define ULTRA_MIXER_STEREODEVS	_IOR ( 'M', 251, int )
#define ULTRA_MIXER_OSS_INFO    _IOR ( 'M', 101, struct oss_mixer_info )

#define ULTRA_MIXER_PRI_MASTER		0x00000100
#define ULTRA_MIXER_PRI_SYNTHESIZER	0x00000200
#define ULTRA_MIXER_PRI_SYNTHESIZER1	0x00000300
#define ULTRA_MIXER_PRI_EFFECT		0x00000400
#define ULTRA_MIXER_PRI_PCM		0x00000500
#define ULTRA_MIXER_PRI_PCM1		0x00000600
#define ULTRA_MIXER_PRI_LINE		0x00000700
#define ULTRA_MIXER_PRI_MIC		0x00000800
#define ULTRA_MIXER_PRI_CD		0x00000900
#define ULTRA_MIXER_PRI_GAIN		0x00000a00
#define ULTRA_MIXER_PRI_IGAIN		0x00000b00
#define ULTRA_MIXER_PRI_OGAIN		0x00000c00
#define ULTRA_MIXER_PRI_LOOPBACK	0x00000d00
#define ULTRA_MIXER_PRI_SPEAKER		0x00000e00
#define ULTRA_MIXER_PRI_AUXA		0xf0000000
#define ULTRA_MIXER_PRI_AUXB		0xf0000100
#define ULTRA_MIXER_PRI_AUXC		0xf0000200

#define ULTRA_MIX_MUTE_LEFT	1
#define ULTRA_MIX_MUTE_RIGHT	2
#define ULTRA_MIX_MUTE		(ULTRA_MIX_MUTE_LEFT|ULTRA_MIX_MUTE_RIGHT)

#define ULTRA_MIX_FORCE		0x80000000

struct ULTRA_STRU_MIXER_CHANNEL;
struct ULTRA_STRU_MIXER_FILE;
struct ULTRA_STRU_MIXER;

typedef struct ULTRA_STRU_MIXER_CHANNEL ultra_mixer_channel_t;
typedef struct ULTRA_STRU_MIXER_FILE ultra_mixer_file_t;
typedef struct ULTRA_STRU_MIXER ultra_mixer_t;

struct ULTRA_STRU_MIXER_CHANNEL_HW {
  unsigned int priority;	/* this is sorting key - highter value = end */
  char name[ 16 ];		/* device name */
  unsigned short ossdev;	/* assigned OSS device number */
  unsigned short mute: 1,	/* mute is supported */
                 stereo: 1,	/* stereo is supported */
                 record: 1;	/* record is supported */
  unsigned int max;		/* max left & right value (from 0) */
  unsigned int private_value;	/* can be used by low-level driver */

  /* input = 0 to 100 (user value), output = 0 to max (low-level value) */
  /* NULL -> linear step */
  unsigned int (*compute_volume)( unsigned int volume );
  void (*set_record_source)( ultra_mixer_t *mixer, ultra_mixer_channel_t *channel, int enable );
  void (*set_mute)( ultra_mixer_t *mixer, ultra_mixer_channel_t *channel, unsigned int mute );
  void (*set_volume_level)( ultra_mixer_t *mixer, ultra_mixer_channel_t *channel, unsigned int left, unsigned int right );
};

struct ULTRA_STRU_MIXER_CHANNEL {
  unsigned short channel;	/* channel index */
  unsigned short record: 1;	/* recording is enabled */
 
  unsigned char umute;		/* user mute */
  unsigned char kmute;		/* kernel mute */

  unsigned char mute;		/* real mute flags */
  unsigned char pad;		/* reserved */

  unsigned char aleft;		/* application - 0 to 100 */
  unsigned char aright;		/* application - 0 to 100 */
  unsigned int uleft;		/* user - 0 to max */
  unsigned int uright;		/* user - 0 to max */
  unsigned int left;		/* real - 0 to max */
  unsigned int right;		/* real - 0 to max */

  unsigned int private_data;
  
  struct ULTRA_STRU_MIXER_CHANNEL_HW hw; /* readonly variables */ 
};

struct ULTRA_STRU_MIXER_FILE {
  ultra_mixer_t *mixer;
  unsigned int changes;
  SLEEP_DEFINE( change );
  struct ULTRA_STRU_MIXER_FILE *next;
};

struct ULTRA_STRU_MIXER_HW {
  unsigned int caps;
  int (*set_special)( ultra_mixer_t *mixer, struct ULTRA_MIXER_SPECIAL *special );
  int (*get_special)( ultra_mixer_t *mixer, struct ULTRA_MIXER_SPECIAL *special );
};

struct ULTRA_STRU_MIXER {
  ultra_card_t *card;

  char id[ 32 ];
  unsigned char info[ 64 ];

  unsigned short port;

  unsigned int channels_count;		/* channels count */
  ultra_mixer_channel_t *channels[ ULTRA_MIXER_DEVS ];

  void *private_data;
  void (*private_free)( void *private_data );
  unsigned int private_value;

  struct ULTRA_STRU_MIXER_HW hw; 	/* readonly variables */

  ultra_mixer_file_t *ffile;		/* first file */
  MUTEX_DEFINE( ffile );
};

extern void ultra_mixer_set_kernel_mute( ultra_mixer_t *mixer, unsigned int priority, unsigned short mute );

extern ultra_mixer_t *ultra_mixer_new( ultra_card_t *card, char *id, char *info, unsigned short port );
extern int ultra_mixer_free( ultra_mixer_t *mixer );

extern ultra_mixer_channel_t *ultra_mixer_new_channel( ultra_mixer_t *mixer, struct ULTRA_STRU_MIXER_CHANNEL_HW *hw );
extern void ultra_mixer_reorder_channel( ultra_mixer_t *mixer, ultra_mixer_channel_t *channel );

extern int ultra_mixer_register( ultra_mixer_t *mixer, int device );
extern int ultra_mixer_unregister( ultra_mixer_t *mixer );

extern int ultra_mixer_channel_init( ultra_mixer_t *mixer, unsigned int priority, unsigned char left, unsigned char right, unsigned int flags );
